@@ -1082,7 +1082,7 @@ create_directory_get_filedir_parms_cb (GObject *source_object, GAsyncResult *res
   return;
 
 error:
-  g_clear_object (info);
+  g_clear_object (&info);
   g_simple_async_result_take_error (simple, err);
   g_simple_async_result_complete (simple);
   g_object_unref (simple);
@@ -3033,4 +3033,4 @@ g_vfs_afp_volume_read_from_fork_finish (GVfsAfpVolume  *volume,
     *bytes_read = g_simple_async_result_get_op_res_gssize (simple);
   
   return TRUE;
-}
\ No newline at end of file
+}
