@@ -58,96 +58,35 @@ struct _GVfsBackendAfpBrowse
   GVfsAfpServer      *server;
 
   char               *logged_in_user;
-  GSList             *volumes;
+  GPtrArray          *volumes;
 };
 
 
 G_DEFINE_TYPE (GVfsBackendAfpBrowse, g_vfs_backend_afp_browse, G_VFS_TYPE_BACKEND);
 
 
-typedef struct
-{
-  char *name;
-  guint16 flags;
-} VolumeData;
-
 static void
-volume_data_free (VolumeData *vol_data)
+get_volumes_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
-  g_free (vol_data->name);
-  g_slice_free (VolumeData, vol_data);
-}
-
-static gboolean
-is_root (const char *filename)
-{
-  const char *p;
-
-  p = filename;
-  while (*p == '/')
-    p++;
-
-  return *p == 0;
-}
-
-static void
-get_srvr_parms_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
-{
-  GVfsAfpConnection *afp_conn = G_VFS_AFP_CONNECTION (source_object);
-  GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (user_data);
+  GVfsAfpServer *server = G_VFS_AFP_SERVER (source_object);
+  GSimpleAsyncResult *simple = user_data;
 
   GVfsBackendAfpBrowse *afp_backend;
-  GVfsAfpReply *reply;
+  GPtrArray *volumes;
   GError *err = NULL;
-  AfpResultCode res_code;
   
-  guint8 num_volumes, i;
-
   afp_backend = G_VFS_BACKEND_AFP_BROWSE (g_async_result_get_source_object (G_ASYNC_RESULT (simple)));
   
-  reply = g_vfs_afp_connection_send_command_finish (afp_conn, res, &err);
-  if (!reply)
+  volumes = g_vfs_afp_server_get_volumes_finish (server, res, &err);
+  if (!volumes)
   {
     g_simple_async_result_take_error (simple, err);
     goto done;
   }
 
-  res_code = g_vfs_afp_reply_get_result_code (reply);
-  if (res_code != AFP_RESULT_NO_ERROR)
-  {
-    g_object_unref (reply);
-
-    g_simple_async_result_set_error (simple, G_IO_ERROR, G_IO_ERROR_FAILED,
-                                     _("Got error code: %d from server"), res_code);
-    goto done;
-  }
-  
-  /* server time */
-  g_vfs_afp_reply_read_int32 (reply, NULL);
-
-  g_slist_free_full (afp_backend->volumes, (GDestroyNotify) volume_data_free);
-  afp_backend->volumes = NULL;
-
-  g_vfs_afp_reply_read_byte (reply, &num_volumes);
-  for (i = 0; i < num_volumes; i++)
-  {
-    guint8 flags;
-    char *vol_name;
-
-    VolumeData *volume_data;
-
-    g_vfs_afp_reply_read_byte (reply, &flags);
-    g_vfs_afp_reply_read_pascal (reply, &vol_name);
-    if (!vol_name)
-      continue;
-
-    volume_data = g_slice_new (VolumeData);
-    volume_data->flags = flags;
-    volume_data->name = vol_name;
-
-    afp_backend->volumes = g_slist_prepend (afp_backend->volumes, volume_data);
-  }
-  g_object_unref (reply);
+  if (afp_backend->volumes)
+    g_ptr_array_unref (afp_backend->volumes);
+  afp_backend->volumes = volumes;
 
 done:
   g_simple_async_result_complete (simple);
@@ -160,20 +99,13 @@ update_cache (GVfsBackendAfpBrowse *afp_backend,
               GAsyncReadyCallback callback,
               gpointer user_data)
 {
-  GVfsAfpCommand *comm;
   GSimpleAsyncResult *simple;
   
-  comm = g_vfs_afp_command_new (AFP_COMMAND_GET_SRVR_PARMS);
-  /* pad byte */
-  g_vfs_afp_command_put_byte (comm, 0);
-
   simple = g_simple_async_result_new (G_OBJECT (afp_backend), callback,
                                       user_data, update_cache);
-  
-  g_vfs_afp_connection_send_command (afp_backend->server->conn, comm, NULL,
-                                     get_srvr_parms_cb,
-                                     cancellable, simple);
-  g_object_unref (comm); 
+
+  g_vfs_afp_server_get_volumes (afp_backend->server, cancellable, get_volumes_cb,
+                                simple); 
 }
 
 static gboolean
@@ -195,13 +127,13 @@ update_cache_finish (GVfsBackendAfpBrowse *afp_backend,
   return TRUE;
 }
 
-static VolumeData *
+static GVfsAfpVolumeData *
 find_volume (GVfsBackendAfpBrowse *afp_backend,
              char *filename)
 {
   char *end;
   guint len;
-  GSList *l;
+  guint i;
 
   while (*filename == '/')
     filename++;
@@ -220,9 +152,9 @@ find_volume (GVfsBackendAfpBrowse *afp_backend,
   else
     len = strlen (filename);
 
-  for (l = afp_backend->volumes; l; l = g_slist_next (l))
+  for (i = 0; i < afp_backend->volumes->len; i++)
   {
-    VolumeData *vol_data = (VolumeData *)l->data;
+    GVfsAfpVolumeData *vol_data = g_ptr_array_index (afp_backend->volumes, i);
 
     if (strlen (vol_data->name) == len && strncmp (vol_data->name, filename, len) == 0)
       return vol_data;
@@ -240,7 +172,7 @@ mount_mountable_cb (GObject      *source_object,
   GVfsJobMountMountable *job = G_VFS_JOB_MOUNT_MOUNTABLE (user_data);
 
   GError *err;
-  VolumeData *vol_data;
+  GVfsAfpVolumeData *vol_data;
   GMountSpec *mount_spec;
 
   if (!update_cache_finish (afp_backend, res, &err))
@@ -293,7 +225,7 @@ try_mount_mountable (GVfsBackend *backend,
 }
 
 static void
-fill_info (GFileInfo *info, VolumeData *vol_data, GVfsBackendAfpBrowse *afp_backend)
+fill_info (GFileInfo *info, GVfsAfpVolumeData *vol_data, GVfsBackendAfpBrowse *afp_backend)
 {
   GIcon *icon;
   GMountSpec *mount_spec;
@@ -347,7 +279,7 @@ enumerate_cache_updated_cb (GObject      *source_object,
   GVfsJobEnumerate *job = G_VFS_JOB_ENUMERATE (user_data);
 
   GError *err = NULL;
-  GSList *l;
+  guint i;
 
   if (!update_cache_finish (afp_backend, res, &err))
   {
@@ -358,9 +290,9 @@ enumerate_cache_updated_cb (GObject      *source_object,
 
   g_vfs_job_succeeded (G_VFS_JOB (job));
   
-  for (l = afp_backend->volumes; l; l = l->next)
+  for (i = 0; i < afp_backend->volumes->len; i++)
   {
-    VolumeData *vol_data = l->data;
+    GVfsAfpVolumeData *vol_data = g_ptr_array_index (afp_backend->volumes, i);
     
     GFileInfo *info;
 
@@ -406,7 +338,7 @@ query_info_cb (GObject      *source_object,
   GVfsJobQueryInfo *job = G_VFS_JOB_QUERY_INFO (user_data);
 
   GError *err = NULL;
-  VolumeData *vol_data;
+  GVfsAfpVolumeData *vol_data;
 
   if (!update_cache_finish (afp_backend, res, &err))
   {
@@ -495,10 +427,12 @@ do_mount (GVfsBackend *backend,
     server_name = afp_backend->server->server_name;
   
   if (afp_backend->user)
-    display_name = g_strdup_printf (_("AFP shares for %s on %s"), afp_backend->user,
+    /* Translators: first %s is username and second serververname */
+    display_name = g_strdup_printf (_("AFP volumes for %s on %s"), afp_backend->user,
                                     server_name);
   else
-    display_name = g_strdup_printf (_("AFP shares on %s"),
+    /* Translators: %s is the servername */
+    display_name = g_strdup_printf (_("AFP volumes on %s"),
                                     server_name);
   g_vfs_backend_set_display_name (backend, display_name);
   g_free (display_name);
@@ -576,7 +510,8 @@ g_vfs_backend_afp_browse_finalize (GObject *object)
   g_free (afp_backend->user);
 
   g_free (afp_backend->logged_in_user);
-  g_slist_free_full (afp_backend->volumes, (GDestroyNotify)volume_data_free);
+  if (afp_backend->volumes)
+    g_ptr_array_unref (afp_backend->volumes);
 
   G_OBJECT_CLASS (g_vfs_backend_afp_browse_parent_class)->finalize (object);
 }
