@@ -846,7 +846,9 @@ umount_command_cb (GObject       *source_object,
       goto out;
     }
 
-  if (standard_error != NULL && strstr (standard_error, "device is busy") != NULL)
+  if (standard_error != NULL &&
+      (strstr (standard_error, "device is busy") != NULL ||
+       strstr (standard_error, "target is busy") != NULL))
     {
       unmount_show_busy (data, data->mount->mount_path);
       goto out;
@@ -899,6 +901,7 @@ unmount_do (UnmountData *data,
                              "{sv}",
                              "force", g_variant_new_boolean (TRUE));
     }
+  g_dbus_proxy_set_default_timeout (G_DBUS_PROXY (data->filesystem), G_MAXINT);
   udisks_filesystem_call_unmount (data->filesystem,
                                   g_variant_builder_end (&builder),
                                   data->cancellable,
